/*
 * Name: gui.h
 *
 * Purpose: Encapsulate GUI-related data structures and functions
 *
 * History: Adapted from types.h and proto.h of the Matthew Chapman's rdesktop code.
 *
 * Author: Luke Scharf (luke@vt.edu)
 */


/* Multiple include protection */
#ifndef _GUI_H_
#define _GUI_H_


/* Includes */
#include <gtk/gtk.h>
#include "types.h"


/* Defines */
#define GUI_DEFAULTWIDTH 800
#define GUI_DEFAULTHEIGHT 600
#define GUI_GTK_MINCOLORS 256
#define GUI_GTK_LINEWIDTH 1


/* Typedefs */
typedef void* HBITMAP;


/* Misc Structures */
typedef struct _FONTGLYPH
{
	uint16 offset;
	uint16 baseline;
	uint16 width;
	uint16 height;
	void*  pixmap;
} FONTGLYPH;
typedef void* HGLYPH;


typedef struct _BOUNDS
{
	uint16 left;
	uint16 top;
	uint16 right;
	uint16 bottom;

} BOUNDS;


/* Colour structures */
typedef struct _PEN
{
	uint8 style;
	uint8 width;
	uint8 colour;

} PEN;

typedef struct _BRUSH
{
	uint8 xorigin;
	uint8 yorigin;
	uint8 style;
	uint8 pattern[8];

} BRUSH;

typedef struct _COLOURENTRY
{
	uint8 red;
	uint8 green;
	uint8 blue;

} COLOURENTRY;


typedef struct _COLOURMAP
{
	uint16 ncolours;
	COLOURENTRY *colours;

} COLOURMAP;

typedef struct _GTKCOLORMAP
{
	GdkColor** colors;
	int ncolors;

	GdkColormap* colormap;
	GdkGC* gc;
} GTKCOLORMAP;
typedef GTKCOLORMAP* HCOLOURMAP;




/* GUI Functions */
void ui_init(int* argc_p, char*** argv_p);
void ui_exit();
BOOL ui_create_window(char *title);
void ui_destroy_window(void);
void ui_process_events(void);
void ui_move_pointer(int x, int y);
HBITMAP ui_create_bitmap(int width, int height, uint8 *data);
void ui_paint_bitmap(int x, int y, int cx, int cy, int width, int height, uint8 *data);
void ui_destroy_bitmap(HBITMAP bmp);
HGLYPH ui_create_glyph(int width, int height, uint8 *data);
void ui_destroy_glyph(HGLYPH glyph);
HCOLOURMAP ui_create_colourmap(COLOURMAP *colours);
void ui_destroy_colourmap(HCOLOURMAP map);
void ui_set_colourmap(HCOLOURMAP map);
void ui_set_clip(int x, int y, int cx, int cy);
void ui_reset_clip(void);
void ui_bell(void);
void ui_destblt(uint8 opcode, int x, int y, int cx, int cy);
void ui_patblt(uint8 opcode, int x, int y, int cx, int cy, BRUSH *brush, int bgcolour, int fgcolour);
void ui_screenblt(uint8 opcode, int x, int y, int cx, int cy, int srcx, int srcy);
void ui_memblt(uint8 opcode, int x, int y, int cx, int cy, HBITMAP src, int srcx, int srcy);
void ui_triblt(uint8 opcode, int x, int y, int cx, int cy, HBITMAP src, int srcx, int srcy, BRUSH *brush, int bgcolour, int fgcolour);
void ui_line(uint8 opcode, int startx, int starty, int endx, int endy, PEN *pen);
void ui_rect(int x, int y, int cx, int cy, int colour);
void ui_draw_glyph(int mixmode, int x, int y, int cx, int cy, HGLYPH glyph, int srcx, int srcy, int bgcolour, int fgcolour);
void ui_draw_text(uint8 font, uint8 flags, int mixmode, int x, int y, int clipx, int clipy, int clipcx, int clipcy, int boxx, int boxy, int boxcx, int boxcy, int bgcolour, int fgcolour, uint8 *text, uint8 length);
void ui_desktop_save(uint32 offset, int x, int y, int cx, int cy);
void ui_desktop_restore(uint32 offset, int x, int y, int cx, int cy);



/* Multiple include protection */
#endif
